/*
** Generated by X-Designer
*/
#ifndef _uisuites_h
#define _uisuites_h

#define XD_MOTIF

#include <xdclass.h>

class suites_form_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget suites_form;
	Widget list_;
	Widget tools_;
	Widget apply_;
public:
	static void tellCB( Widget, XtPointer, XtPointer );
	virtual void tellCB( Widget, XtPointer ) = 0;
	static void offCB( Widget, XtPointer, XtPointer );
	virtual void offCB( Widget, XtPointer ) = 0;
	static void onCB( Widget, XtPointer, XtPointer );
	virtual void onCB( Widget, XtPointer ) = 0;
};

typedef suites_form_c *suites_form_p;


extern suites_form_p suites_form;


#endif
