/*
** Generated by X-Designer
*/
#ifndef _uicollector_h
#define _uicollector_h

#define XD_MOTIF

#include <xdclass.h>

class collector_shell_c: public xd_XmDialog_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget collector_shell;
	Widget form_;
	Widget command_;
	Widget text_;
	Widget list_;
	Widget stop_;
	Widget remove_;
	Widget all_;
	Widget none_;
	Widget blocks_;
public:
	static void stopCB( Widget, XtPointer, XtPointer );
	virtual void stopCB( Widget, XtPointer ) = 0;
	static void entryCB( Widget, XtPointer, XtPointer );
	virtual void entryCB( Widget, XtPointer ) = 0;
	static void closeCB( Widget, XtPointer, XtPointer );
	virtual void closeCB( Widget, XtPointer ) = 0;
	static void selectCB( Widget, XtPointer, XtPointer );
	virtual void selectCB( Widget, XtPointer ) = 0;
	static void applyCB( Widget, XtPointer, XtPointer );
	virtual void applyCB( Widget, XtPointer ) = 0;
	static void removeCB( Widget, XtPointer, XtPointer );
	virtual void removeCB( Widget, XtPointer ) = 0;
	static void noneCB( Widget, XtPointer, XtPointer );
	virtual void noneCB( Widget, XtPointer ) = 0;
	static void allCB( Widget, XtPointer, XtPointer );
	virtual void allCB( Widget, XtPointer ) = 0;
};

typedef collector_shell_c *collector_shell_p;


extern collector_shell_p collector_shell;


#endif
