/////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
// Name        :
// Author      : Avi
// Revision    : $Revision: #89 $
//
// Copyright 2009-2017 ECMWF.
// This software is licensed under the terms of the Apache Licence version 2.0
// which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
// In applying this licence, ECMWF does not waive the privileges and immunities
// granted to it by virtue of its status as an intergovernmental organisation
// nor does it submit to any jurisdiction.
//
// Description :
/////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8

#include "GlossaryDoc.hpp"

const char* GlossaryDoc ::list(){
   return
            "Create a empty Defs\n\n"
            ".. _active: glossary_\n"
            ".. _aborted: glossary_\n"
            ".. _autocancel: glossary_\n"
            ".. _complete expression: glossary_\n"
            ".. _check point: glossary_\n"
            ".. _child command: glossary_\n"
            ".. _clock: glossary_\n"
            ".. _complete: glossary_\n"
            ".. _cron: glossary_\n"
            ".. _date: glossary_\n"
            ".. _day: glossary_\n"
            ".. _defstatus: glossary_\n"
            ".. _dependencies: glossary_\n"
            ".. _ecflow_server: glossary_\n"
            ".. _ecf script: glossary_\n"
            ".. _event: glossary_\n"
            ".. _extern: glossary_\n"
            ".. _family: glossary_\n"
            ".. _halted: glossary_\n"
            ".. _inlimit: glossary_\n"
            ".. _job creation: glossary_\n"
            ".. _job file: glossary_\n"
            ".. _label: glossary_\n"
            ".. _late: glossary_\n"
            ".. _limit: glossary_\n"
            ".. _meter: glossary_\n"
            ".. _node: glossary_\n"
            ".. _pre-processing: glossary_\n"
            ".. _repeat: glossary_\n"
            ".. _server states: glossary_\n"
            ".. _suite definition: glossary_\n"
            ".. _submitted: glossary_\n"
            ".. _suite: glossary_\n"
            ".. _shutdown: glossary_\n"
            ".. _suspended: glossary_\n"
            ".. _task: glossary_\n"
            ".. _time: glossary_\n"
            ".. _today: glossary_\n"
            ".. _trigger: glossary_\n"
            ".. _queued: glossary_\n"
            ".. _unknown: glossary_\n"
            ".. _variable inheritance: glossary_\n"
            ".. _variable substitution: glossary_\n"
            ".. _variable: glossary_\n"
            ".. _zombie: glossary_\n"
            ".. _zombie type: glossary_\n"
            ".. _glossary: https://software.ecmwf.int/wiki/display/ECFLOW/Glossary\n"
            ;
}

