//=============================================================================================
// Name        : 
// Author      : 
// Revision    : $Revision: #7 $ 
//
// Copyright 2009-2017 ECMWF.
// This software is licensed under the terms of the Apache Licence version 2.0 
// which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
// In applying this licence, ECMWF does not waive the privileges and immunities 
// granted to it by virtue of its status as an intergovernmental organisation 
// nor does it submit to any jurisdiction. 
//
// Description : 
//=============================================================================================

#include "late_node.h"

xmstring late_node::make_label_tree()
{
  if (!owner_) return xmstring("late: ","bold") + xmstring(label_.c_str());
  return xmstring(label_.c_str());
}

void late_node::perlify(FILE* f)
{
  perl_member(f,"value",label_.c_str());
}

const std::string& late_node::name() const { return label_; }
