/*
** Generated by X-Designer
*/
#ifndef _uiedit_label_h
#define _uiedit_label_h

#define XD_MOTIF

#include <xdclass.h>

class edit_label_form_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget edit_label_form;
	Widget tools_;
	Widget value_;
	Widget default_;
public:
	static void changedCB( Widget, XtPointer, XtPointer );
	virtual void changedCB( Widget, XtPointer ) = 0;
	static void applyCB( Widget, XtPointer, XtPointer );
	virtual void applyCB( Widget, XtPointer ) = 0;
};

typedef edit_label_form_c *edit_label_form_p;
class edit_label;
typedef edit_label *edit_label_p;


extern edit_label_p edit_label_form;


#endif
