/*
** Generated by X-Designer
*/
#ifndef _uiedit_limit_h
#define _uiedit_limit_h

#define XD_MOTIF

#include <xdclass.h>

class edit_limit_form_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget edit_limit_form;
	Widget max_;
	Widget tools_;
	Widget list_;
	Widget remove_;
public:
	static void browseCB( Widget, XtPointer, XtPointer );
	virtual void browseCB( Widget, XtPointer ) = 0;
	static void removeCB( Widget, XtPointer, XtPointer );
	virtual void removeCB( Widget, XtPointer ) = 0;
	static void changedCB( Widget, XtPointer, XtPointer );
	virtual void changedCB( Widget, XtPointer ) = 0;
	static void applyCB( Widget, XtPointer, XtPointer );
	virtual void applyCB( Widget, XtPointer ) = 0;
};

typedef edit_limit_form_c *edit_limit_form_p;
class edit_limit;
typedef edit_limit *edit_limit_p;


extern edit_limit_p edit_limit_form;


#endif
