/*
** Generated by X-Designer
*/
/*
**LIBS: -lXm -lXt -lX11
*/

#include <stdlib.h>
#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#include <Xm/Form.h>
#include <Xm/List.h>
#include <Xm/ScrollBar.h>


#include "uihistory.h"

history_p history_form = (history_p) NULL;



void history_form_c::create (Widget parent, char *widget_name)
{
  /* Widget children[3];      // Children to manage */
	Arg al[64];                    /* Arg List */
	register int ac = 0;           /* Arg Count */
	Widget scrolledList9 = (Widget)NULL;

	if ( !widget_name )
		widget_name = "history_form";

	XtSetArg(al[ac], XmNautoUnmanage, FALSE); ac++;
	history_form = XmCreateForm ( parent, widget_name, al, ac );
	ac = 0;
	_xd_rootwidget = history_form;
	XtSetArg(al[ac], XmNlistSpacing, 2); ac++;
	XtSetArg(al[ac], XmNlistSizePolicy, XmCONSTANT); ac++;
	list_ = XmCreateScrolledList ( history_form, "list_", al, ac );
	ac = 0;
	scrolledList9 = XtParent ( list_ );


	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( scrolledList9,al, ac );
	XtAddCallback (list_, XmNbrowseSelectionCallback,&history_form_c:: browseCB, (XtPointer) this);
	XtManageChild(list_);
}

void history_form_c::browseCB( Widget widget, XtPointer client_data, XtPointer call_data )
{
	history_form_p instance = (history_form_p) client_data;
	instance->browseCB ( widget, call_data );
}

