/*
** Generated by X-Designer
*/
/*
**LIBS: -lXm -lXt -lX11
*/

#include <stdlib.h>
#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/Separator.h>
#include <Xm/TextF.h>
#include <Xm/LabelG.h>
#include <Xm/ToggleBG.h>


#include "uifonts.h"

fonts_form_p fonts_form = (fonts_form_p) NULL;



void fonts_form_c::create (Widget parent, char *widget_name)
{
	Widget children[4];      /* Children to manage */
	Arg al[64];                    /* Arg List */
	register int ac = 0;           /* Arg Count */
	Widget frame2 = (Widget)NULL;
	Widget label2 = (Widget)NULL;
	Widget form1 = (Widget)NULL;
	Widget form2 = (Widget)NULL;
	Widget toggle2 = (Widget)NULL;
	Widget separator7 = (Widget)NULL;
	Widget label7 = (Widget)NULL;
	Widget text1 = (Widget)NULL;
	Widget form3 = (Widget)NULL;
	Widget toggle8 = (Widget)NULL;
	Widget separator8 = (Widget)NULL;
	Widget label11 = (Widget)NULL;
	Widget text5 = (Widget)NULL;
	Widget form4 = (Widget)NULL;
	Widget toggle9 = (Widget)NULL;
	Widget separator9 = (Widget)NULL;
	Widget label12 = (Widget)NULL;
	Widget text6 = (Widget)NULL;
	Widget form5 = (Widget)NULL;
	Widget toggle10 = (Widget)NULL;
	Widget separator10 = (Widget)NULL;
	Widget label13 = (Widget)NULL;
	Widget text7 = (Widget)NULL;
	Widget label3 = (Widget)NULL;

	if ( !widget_name )
		widget_name = "Fonts";

	XtSetArg(al[ac], XmNautoUnmanage, FALSE); ac++;
	fonts_form = XmCreateForm ( parent, widget_name, al, ac );
	ac = 0;
	_xd_rootwidget = fonts_form;
	XtSetArg(al[ac], XmNautoUnmanage, FALSE); ac++;
	form_ = XmCreateForm ( fonts_form, "form_", al, ac );
	ac = 0;
	frame2 = XmCreateFrame ( form_, "frame2", al, ac );
	XtSetArg(al[ac], XmNchildType, XmFRAME_TITLE_CHILD); ac++;
	label2 = XmCreateLabel ( frame2, "Fonts", al, ac );
	ac = 0;
	form1 = XmCreateForm ( frame2, "form1", al, ac );
	form2 = XmCreateForm ( form1, "form2", al, ac );
	XtSetArg(al[ac], XmNindicatorType, XmONE_OF_MANY); ac++;
	XtSetArg(al[ac], XmNset, FALSE); ac++;
	toggle2 = XmCreateToggleButtonGadget ( form2, "@normal_font_plain", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNorientation, XmVERTICAL); ac++;
	separator7 = XmCreateSeparator ( form2, "separator7", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNwidth, 100); ac++;
	XtSetArg(al[ac], XmNalignment, XmALIGNMENT_END); ac++;
	XtSetArg(al[ac], XmNrecomputeSize, FALSE); ac++;
	label7 = XmCreateLabel ( form2, "Normal font, plain", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNcolumns, 40); ac++;
	text1 = XmCreateTextField ( form2, "normal_font_plain", al, ac );
	ac = 0;
	form3 = XmCreateForm ( form1, "form3", al, ac );
	XtSetArg(al[ac], XmNindicatorType, XmONE_OF_MANY); ac++;
	XtSetArg(al[ac], XmNset, FALSE); ac++;
	toggle8 = XmCreateToggleButtonGadget ( form3, "@normal_font_bold", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNorientation, XmVERTICAL); ac++;
	separator8 = XmCreateSeparator ( form3, "separator8", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNwidth, 100); ac++;
	XtSetArg(al[ac], XmNalignment, XmALIGNMENT_END); ac++;
	XtSetArg(al[ac], XmNrecomputeSize, FALSE); ac++;
	label11 = XmCreateLabel ( form3, "Normal font, bold", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNcolumns, 40); ac++;
	text5 = XmCreateTextField ( form3, "normal_font_bold", al, ac );
	ac = 0;
	form4 = XmCreateForm ( form1, "form4", al, ac );
	XtSetArg(al[ac], XmNindicatorType, XmONE_OF_MANY); ac++;
	XtSetArg(al[ac], XmNset, FALSE); ac++;
	toggle9 = XmCreateToggleButtonGadget ( form4, "@small_font_plain", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNorientation, XmVERTICAL); ac++;
	separator9 = XmCreateSeparator ( form4, "separator9", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNwidth, 100); ac++;
	XtSetArg(al[ac], XmNalignment, XmALIGNMENT_END); ac++;
	XtSetArg(al[ac], XmNrecomputeSize, FALSE); ac++;
	label12 = XmCreateLabel ( form4, "Small font, plain", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNcolumns, 40); ac++;
	text6 = XmCreateTextField ( form4, "small_font_plain", al, ac );
	ac = 0;
	form5 = XmCreateForm ( form1, "form5", al, ac );
	XtSetArg(al[ac], XmNindicatorType, XmONE_OF_MANY); ac++;
	XtSetArg(al[ac], XmNset, FALSE); ac++;
	toggle10 = XmCreateToggleButtonGadget ( form5, "@small_font_bold", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNorientation, XmVERTICAL); ac++;
	separator10 = XmCreateSeparator ( form5, "separator10", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNwidth, 100); ac++;
	XtSetArg(al[ac], XmNalignment, XmALIGNMENT_END); ac++;
	XtSetArg(al[ac], XmNrecomputeSize, FALSE); ac++;
	label13 = XmCreateLabel ( form5, "Small font, bold", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNcolumns, 40); ac++;
	text7 = XmCreateTextField ( form5, "small_font_bold", al, ac );
	ac = 0;
	label3 = XmCreateLabelGadget ( form_, "warn", al, ac );

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( form_,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopOffset, 5); ac++;
	XtSetArg(al[ac], XmNtopWidget, label3); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftOffset, 5); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightOffset, 5); ac++;
	XtSetValues ( frame2,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNtopOffset, 5); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftOffset, 5); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightOffset, 5); ac++;
	XtSetValues ( label3,al, ac );
	ac = 0;


	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightOffset, 0); ac++;
	XtSetValues ( form2,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget, form2); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightOffset, 0); ac++;
	XtSetValues ( form3,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget, form3); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightOffset, 0); ac++;
	XtSetValues ( form4,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget, form4); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightOffset, 0); ac++;
	XtSetValues ( form5,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_NONE); ac++;
	XtSetValues ( toggle2,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNtopOffset, 4); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftWidget, toggle2); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_NONE); ac++;
	XtSetValues ( separator7,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftWidget, separator7); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_NONE); ac++;
	XtSetValues ( label7,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftWidget, label7); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( text1,al, ac );
	ac = 0;
	XtAddCallback (toggle2, XmNvalueChangedCallback,&fonts_form_c:: useCB, (XtPointer) this);
	XtAddCallback (text1, XmNactivateCallback,&fonts_form_c:: changedCB, (XtPointer) this);
	children[ac++] = toggle2;
	children[ac++] = separator7;
	children[ac++] = label7;
	children[ac++] = text1;
	XtManageChildren(children, ac);
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomOffset, 0); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( toggle8,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNtopOffset, 4); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftWidget, toggle8); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_NONE); ac++;
	XtSetValues ( separator8,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNtopOffset, 0); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomOffset, 0); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftWidget, separator8); ac++;
	XtSetValues ( label11,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNtopOffset, 0); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomOffset, 0); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 0); ac++;
	XtSetArg(al[ac], XmNleftWidget, label11); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightOffset, 0); ac++;
	XtSetValues ( text5,al, ac );
	ac = 0;
	XtAddCallback (toggle8, XmNvalueChangedCallback,&fonts_form_c:: useCB, (XtPointer) this);
	XtAddCallback (text5, XmNactivateCallback,&fonts_form_c:: changedCB, (XtPointer) this);
	children[ac++] = toggle8;
	children[ac++] = separator8;
	children[ac++] = label11;
	children[ac++] = text5;
	XtManageChildren(children, ac);
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomOffset, 0); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( toggle9,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNtopOffset, 4); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftWidget, toggle9); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_NONE); ac++;
	XtSetValues ( separator9,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNtopOffset, 0); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomOffset, 0); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftWidget, separator9); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_NONE); ac++;
	XtSetValues ( label12,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNtopOffset, 0); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomOffset, 0); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 0); ac++;
	XtSetArg(al[ac], XmNleftWidget, label12); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightOffset, 0); ac++;
	XtSetValues ( text6,al, ac );
	ac = 0;
	XtAddCallback (toggle9, XmNvalueChangedCallback,&fonts_form_c:: useCB, (XtPointer) this);
	XtAddCallback (text6, XmNactivateCallback,&fonts_form_c:: changedCB, (XtPointer) this);
	children[ac++] = toggle9;
	children[ac++] = separator9;
	children[ac++] = label12;
	children[ac++] = text6;
	XtManageChildren(children, ac);
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNtopOffset, 0); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomOffset, 0); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftOffset, 0); ac++;
	XtSetValues ( toggle10,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNtopOffset, 4); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftWidget, toggle10); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_NONE); ac++;
	XtSetValues ( separator10,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNtopOffset, 0); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomOffset, 0); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftWidget, separator10); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_NONE); ac++;
	XtSetValues ( label13,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNtopOffset, 0); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomOffset, 0); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 0); ac++;
	XtSetArg(al[ac], XmNleftWidget, label13); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightOffset, 0); ac++;
	XtSetValues ( text7,al, ac );
	ac = 0;
	XtAddCallback (toggle10, XmNvalueChangedCallback,&fonts_form_c:: useCB, (XtPointer) this);
	XtAddCallback (text7, XmNactivateCallback,&fonts_form_c:: changedCB, (XtPointer) this);
	children[ac++] = toggle10;
	children[ac++] = separator10;
	children[ac++] = label13;
	children[ac++] = text7;
	XtManageChildren(children, ac);
	ac = 0;
	children[ac++] = form2;
	children[ac++] = form3;
	children[ac++] = form4;
	children[ac++] = form5;
	XtManageChildren(children, ac);
	ac = 0;
	children[ac++] = label2;
	children[ac++] = form1;
	XtManageChildren(children, ac);
	ac = 0;
	children[ac++] = frame2;
	children[ac++] = label3;
	XtManageChildren(children, ac);
	ac = 0;
	children[ac++] = form_;
	XtManageChildren(children, ac);
}

void fonts_form_c::useCB( Widget widget, XtPointer client_data, XtPointer call_data )
{
	fonts_form_p instance = (fonts_form_p) client_data;
	instance->useCB ( widget, call_data );
}

void fonts_form_c::changedCB( Widget widget, XtPointer client_data, XtPointer call_data )
{
	fonts_form_p instance = (fonts_form_p) client_data;
	instance->changedCB ( widget, call_data );
}

