/*
** Generated by X-Designer
*/
#ifndef _uioption_h
#define _uioption_h

#define XD_MOTIF

#include <xdclass.h>

class option_form_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget option_form;
	Widget form_;
	Widget aborted;
	Widget restarted;
	Widget late;

	Widget zombied;
	Widget aliases;
	Widget late_family;

	Widget new_suites;
	Widget direct_read;
	Widget poll;
	Widget timeout;
	Widget drift;
	Widget maximum;
	Widget jobfile_length;
public:
	static void useCB( Widget, XtPointer, XtPointer );
	virtual void useCB( Widget, XtPointer ) = 0;
	static void changedCB( Widget, XtPointer, XtPointer );
	virtual void changedCB( Widget, XtPointer ) = 0;
};

typedef option_form_c *option_form_p;


extern option_form_p option_form;


#endif
