/*
** Generated by X-Designer
*/
#ifndef _uizombies_h
#define _uizombies_h

#define XD_MOTIF

#include <xdclass.h>

class zombies_form_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget zombies_form;
	Widget list_;
	Widget buttons_;
public:
	static void browseCB( Widget, XtPointer, XtPointer );
	virtual void browseCB( Widget, XtPointer ) = 0;
	static void deleteCB( Widget, XtPointer, XtPointer );
	virtual void deleteCB( Widget, XtPointer ) = 0;
	static void acceptCB( Widget, XtPointer, XtPointer );
	virtual void acceptCB( Widget, XtPointer ) = 0;
	static void rescueCB( Widget, XtPointer, XtPointer );
	virtual void rescueCB( Widget, XtPointer ) = 0;
	static void terminateCB( Widget, XtPointer, XtPointer );
	virtual void terminateCB( Widget, XtPointer ) = 0;
	static void killCB( Widget, XtPointer, XtPointer );
	virtual void killCB( Widget, XtPointer ) = 0;
};

typedef zombies_form_c *zombies_form_p;

extern zombies_form_p zombies_form;

#endif
