/*
** Generated by X-Designer
*/
#ifndef _uiscript_h
#define _uiscript_h

#define XD_MOTIF

#include <xdclass.h>

class script_form_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget script_form;
	Widget text_;
	Widget name_;
	Widget tools_;
public:
	static void externalCB( Widget, XtPointer, XtPointer );
	virtual void externalCB( Widget, XtPointer ) = 0;
	static void searchCB( Widget, XtPointer, XtPointer );
	virtual void searchCB( Widget, XtPointer ) = 0;
};

typedef script_form_c *script_form_p;
class script;
typedef script *script_p;


extern script_p script_form;


#endif
