/*
** Generated by X-Designer
*/
#ifndef _uiedit_repeat_h
#define _uiedit_repeat_h

#define XD_MOTIF

#include <xdclass.h>

class edit_repeat_form_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget edit_repeat_form;
	Widget tools_;
	Widget show_list_;
	Widget list_;
	Widget show_text_;
	Widget text_;
	Widget label_;
public:
	static void applyCB( Widget, XtPointer, XtPointer );
	virtual void applyCB( Widget, XtPointer ) = 0;
	static void browseCB( Widget, XtPointer, XtPointer );
	virtual void browseCB( Widget, XtPointer ) = 0;
};

typedef edit_repeat_form_c *edit_repeat_form_p;
class edit_repeat;
typedef edit_repeat *edit_repeat_p;


extern edit_repeat_p edit_repeat_form;


#endif
