#if 1
#include <set>
#include <map>
#include <list>
#include <vector>
#include <string>
#include <algorithm>
using std::string;
using std::set;
using std::map;
using std::list;
using std::vector;
#else
#include <algo.h>
#include <tree.h>
#include <mstring.h>
#include <map.h>
#include <set.h>
#endif
