#include <Xm/Protocols.h>
/*
** Generated by X-Designer
*/
#ifndef _uipanel_h
#define _uipanel_h

#define XD_MOTIF

#include <xdclass.h>

class panel_top_c: public xd_TopLevelShell_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget panel_top;
	Widget form_;
	Widget tab_;
	Widget menubar_;
	Widget file_menu;
	Widget save_;
	Widget print_;
	Widget options_menu;
	Widget detached_;
	Widget frozen_;
	Widget close_on_apply_;
	Widget help_menu;
public:
	static void printCB( Widget, XtPointer, XtPointer );
	virtual void printCB( Widget, XtPointer ) = 0;
	static void saveCB( Widget, XtPointer, XtPointer );
	virtual void saveCB( Widget, XtPointer ) = 0;
	static void resizeCB( Widget, XtPointer, XtPointer );
	virtual void resizeCB( Widget, XtPointer ) = 0;
	static void freezeCB( Widget, XtPointer, XtPointer );
	virtual void freezeCB( Widget, XtPointer ) = 0;
	static void nodeCB( Widget, XtPointer, XtPointer );
	virtual void nodeCB( Widget, XtPointer ) = 0;
	static void cloneCB( Widget, XtPointer, XtPointer );
	virtual void cloneCB( Widget, XtPointer ) = 0;
	static void unmapCB( Widget, XtPointer, XtPointer );
	virtual void unmapCB( Widget, XtPointer ) = 0;
	static void mapCB( Widget, XtPointer, XtPointer );
	virtual void mapCB( Widget, XtPointer ) = 0;
};

typedef panel_top_c *panel_top_p;
class panel_window;
typedef panel_window *panel_window_p;


extern panel_window_p panel_top;


#endif
