suite suiteName
   family familyName0
   		today 15:00
   		today 00:00 23:59 00:30
        today 00:00 06:59 00:30
   endfamily
endsuite

suite s1
   family familyName0
   		today 15:00 ; today 00:00 23:59 00:30
        today 00:00 06:59 00:30
   endfamily
endsuite
