#include <Xm/Protocols.h>
#include "Tab.h"
/*
** Generated by X-Designer
*/
#ifndef _uipref_h
#define _uipref_h

#define XD_MOTIF

#include <xdclass.h>

class pref_shell_c: public xd_XmDialog_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget pref_shell;
	Widget form_;
	Widget tab_;
	Widget button_close;
public:
	static void mapCB( Widget, XtPointer, XtPointer );
	virtual void mapCB( Widget, XtPointer ) = 0;
	static void closeCB( Widget, XtPointer, XtPointer );
	virtual void closeCB( Widget, XtPointer ) = 0;
};

typedef pref_shell_c *pref_shell_p;


extern pref_shell_p pref_shell;


#endif
