//============================================================================
// Name        :
// Author      : Avi
// Revision    : $Revision: #5 $ 
//
// Copyright 2009-2017 ECMWF.
// This software is licensed under the terms of the Apache Licence version 2.0 
// which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
// In applying this licence, ECMWF does not waive the privileges and immunities 
// granted to it by virtue of its status as an intergovernmental organisation 
// nor does it submit to any jurisdiction. 
//
// Description :
//============================================================================

//#include <boost/test/unit_test.hpp>
//#include <string>
//#include <iostream>
//
//
//#include <pwd.h> /* getpwdid */
//#include <sys/types.h>
//#include <unistd.h>
//#include <stdio.h>
//
//using namespace std;
//using namespace ecf;
//
//BOOST_AUTO_TEST_SUITE( CoreTestSuite )

//BOOST_AUTO_TEST_CASE( test_get_user_details )
//{
//	cout << "ACore:: ...test_get_user_details\n";
//
//
//	struct passwd *passwd; /* man getpwuid */
//
//	passwd = getpwuid ( getuid()); /* Get the uid of the running processand use it to get a record from /etc/passwd */
//
//	printf("\n The Real User Name is %s ", passwd->pw_gecos);
//
//	printf("\n The Login Name is %s ", passwd->pw_name);
//
//	printf("\n The Home Directory is %s", passwd->pw_dir);
//
//	printf("\n The Login Shell is %s ", passwd->pw_shell);
//
//	printf("\n The Passwd is %s ", getpwuid(getuid())->pw_passwd);
//
//	printf("\n The uid is %lu ", (unsigned long) getpwuid(getuid())->pw_uid);
//
//	printf("\n The gid is %lu \n\n", (unsigned long) getpwuid(getuid())->pw_gid);
//}

//BOOST_AUTO_TEST_SUITE_END()


