suite s1
    repeat date YMD 20090331 20121212 1 # status 0
endsuite

suite s2
    repeat date YMD 20121212 20090331  -1 # status 0
endsuite

suite s3; repeat date YMD 20090331 20121212 1 # status 0
endsuite