/*
** Generated by X-Designer
*/
#ifndef _uifonts_h
#define _uifonts_h

#define XD_MOTIF

#include <xdclass.h>

class fonts_form_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget fonts_form;
	Widget form_;
public:
	static void useCB( Widget, XtPointer, XtPointer );
	virtual void useCB( Widget, XtPointer ) = 0;
	static void changedCB( Widget, XtPointer, XtPointer );
	virtual void changedCB( Widget, XtPointer ) = 0;
};

typedef fonts_form_c *fonts_form_p;


extern fonts_form_p fonts_form;


#endif
