/*
** Generated by X-Designer
*/
/*
**LIBS: -lXm -lXt -lX11
*/

#include <stdlib.h>
#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/Separator.h>
#include <Xm/TextF.h>
#include <Xm/ToggleB.h>
#include <Xm/LabelG.h>
#include <Xm/ToggleBG.h>


#include "uioption.h"

option_form_p option_form = (option_form_p) NULL;



void option_form_c::create (Widget parent, char *widget_name)
{
	Widget children[10];      /* Children to manage */
	Arg al[64];                    /* Arg List */
	register int ac = 0;           /* Arg Count */
	Widget frame1 = (Widget)NULL;
	Widget label1 = (Widget)NULL;
	Widget form1 = (Widget)NULL;
	Widget form7 = (Widget)NULL;
	Widget toggle1 = (Widget)NULL;
	Widget separator3 = (Widget)NULL;
	Widget form8 = (Widget)NULL;
	Widget toggle4 = (Widget)NULL;
	Widget separator1 = (Widget)NULL;
	Widget separator13 = (Widget)NULL;
	Widget separator14 = (Widget)NULL;
	Widget form9 = (Widget)NULL;
	Widget toggle5 = (Widget)NULL;
	Widget separator2 = (Widget)NULL;
	Widget frame3 = (Widget)NULL;
	Widget label8 = (Widget)NULL;
	Widget form10 = (Widget)NULL;
	Widget form11 = (Widget)NULL;
	Widget toggle3 = (Widget)NULL;
	Widget separator8 = (Widget)NULL;
	Widget form12 = (Widget)NULL;
	Widget form13 = (Widget)NULL;
	Widget form14 = (Widget)NULL;
	Widget toggle10 = (Widget)NULL;
	Widget toggle13 = (Widget)NULL;
	Widget toggle14 = (Widget)NULL;
	Widget separator9 = (Widget)NULL;
	Widget frame2 = (Widget)NULL;
	Widget label2 = (Widget)NULL;
	Widget form2 = (Widget)NULL;
	Widget form4 = (Widget)NULL;
	Widget toggle2 = (Widget)NULL;
	Widget separator7 = (Widget)NULL;
	Widget form6 = (Widget)NULL;
	Widget toggle6 = (Widget)NULL;
	Widget separator4 = (Widget)NULL;
	Widget label3 = (Widget)NULL;
	Widget label5 = (Widget)NULL;
	Widget form5 = (Widget)NULL;
	Widget toggle7 = (Widget)NULL;
	Widget separator5 = (Widget)NULL;
	Widget form3 = (Widget)NULL;
	Widget toggle8 = (Widget)NULL;
	Widget separator6 = (Widget)NULL;
	Widget label4 = (Widget)NULL;
        Widget label6 = (Widget)NULL;
        Widget label7 = (Widget)NULL;

        Widget label20 = (Widget)NULL;
        Widget label21 = (Widget)NULL;
        Widget form21 = (Widget)NULL;
        Widget toggle20 = (Widget)NULL;
        Widget separator20 = (Widget)NULL;

	Widget form15 = (Widget)NULL;
	Widget toggle15 = (Widget)NULL;
	Widget separator15 = (Widget)NULL;

        if ( !widget_name )
                widget_name = "Server options";

	XtSetArg(al[ac], XmNautoUnmanage, FALSE); ac++;
	option_form = XmCreateForm ( parent, widget_name, al, ac );
	ac = 0;
	_xd_rootwidget = option_form;
	XtSetArg(al[ac], XmNautoUnmanage, FALSE); ac++;
	form_ = XmCreateForm ( option_form, "form_", al, ac );
	ac = 0;
	frame1 = XmCreateFrame ( form_, "frame1", al, ac );
	XtSetArg(al[ac], XmNchildType, XmFRAME_TITLE_CHILD); ac++;
	label1 = XmCreateLabel ( frame1, "Automaticaly open these windows:", al, ac );
	ac = 0;
	form1 = XmCreateForm ( frame1, "form1", al, ac );
	form7 = XmCreateForm ( form1, "form7", al, ac );
	XtSetArg(al[ac], XmNindicatorType, XmONE_OF_MANY); ac++;
	XtSetArg(al[ac], XmNset, FALSE); ac++;
	toggle1 = XmCreateToggleButtonGadget ( form7, "@aborted", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNorientation, XmVERTICAL); ac++;
	separator3 = XmCreateSeparator ( form7, "separator3", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNalignment, XmALIGNMENT_BEGINNING); ac++;
	aborted = XmCreateToggleButton ( form7, "aborted", al, ac );
	ac = 0;
	form8 = XmCreateForm ( form1, "form8", al, ac );
	XtSetArg(al[ac], XmNindicatorType, XmONE_OF_MANY); ac++;
	XtSetArg(al[ac], XmNset, FALSE); ac++;
	toggle4 = XmCreateToggleButtonGadget ( form8, "@restarted", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNorientation, XmVERTICAL); ac++;
	separator1 = XmCreateSeparator ( form8, "separator1", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNalignment, XmALIGNMENT_BEGINNING); ac++;
	restarted = XmCreateToggleButton ( form8, "restarted", al, ac );

	ac = 0;
	form9 = XmCreateForm ( form1, "form9", al, ac );
	XtSetArg(al[ac], XmNindicatorType, XmONE_OF_MANY); ac++;
	XtSetArg(al[ac], XmNset, FALSE); ac++;
	toggle5 = XmCreateToggleButtonGadget ( form9, "@late", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNorientation, XmVERTICAL); ac++;
	separator2 = XmCreateSeparator ( form9, "separator2", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNalignment, XmALIGNMENT_BEGINNING); ac++;
	late = XmCreateToggleButton ( form9, "late", al, ac );

	ac = 0;
	form13 = XmCreateForm ( form1, "form13", al, ac );
	XtSetArg(al[ac], XmNindicatorType, XmONE_OF_MANY); ac++;
	XtSetArg(al[ac], XmNset, FALSE); ac++;
	toggle13 = XmCreateToggleButtonGadget ( form13, "@zombied", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNorientation, XmVERTICAL); ac++;
	separator13 = XmCreateSeparator ( form13, "separator1", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNalignment, XmALIGNMENT_BEGINNING); ac++;
	zombied = XmCreateToggleButton ( form13, "zombied", al, ac );

	ac = 0;
	form14 = XmCreateForm ( form1, "form14", al, ac );
	XtSetArg(al[ac], XmNindicatorType, XmONE_OF_MANY); ac++;
	XtSetArg(al[ac], XmNset, FALSE); ac++;
	toggle14 = XmCreateToggleButtonGadget ( form14, "@aliases", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNorientation, XmVERTICAL); ac++;
	separator14 = XmCreateSeparator ( form14, "separator1", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNalignment, XmALIGNMENT_BEGINNING); ac++;
	aliases = XmCreateToggleButton ( form14, "aliases", al, ac );

	/* 20150216 */
	ac = 0;
	form15 = XmCreateForm ( form1, "form15", al, ac );
	XtSetArg(al[ac], XmNindicatorType, XmONE_OF_MANY); ac++;
	XtSetArg(al[ac], XmNset, FALSE); ac++;
	toggle15 = XmCreateToggleButtonGadget ( form15, "@late_family", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNorientation, XmVERTICAL); ac++;
	separator15 = XmCreateSeparator ( form15, "separator1", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNalignment, XmALIGNMENT_BEGINNING); ac++;
	late_family = XmCreateToggleButton ( form15, "late_family", al, ac );

	ac = 0;
	frame3 = XmCreateFrame ( form_, "frame3", al, ac );
	XtSetArg(al[ac], XmNchildType, XmFRAME_TITLE_CHILD); ac++;
	label8 = XmCreateLabel ( frame3, "Miscellaneous:", al, ac );
	ac = 0;
	form10 = XmCreateForm ( frame3, "form10", al, ac );
	form11 = XmCreateForm ( form10, "form11", al, ac );
	XtSetArg(al[ac], XmNindicatorType, XmONE_OF_MANY); ac++;
	XtSetArg(al[ac], XmNset, FALSE); ac++;
	toggle3 = XmCreateToggleButtonGadget ( form11, "@new_suites", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNorientation, XmVERTICAL); ac++;
	separator8 = XmCreateSeparator ( form11, "separator8", al, ac );
        ac = 0;
        XtSetArg(al[ac], XmNalignment, XmALIGNMENT_BEGINNING); ac++;
        new_suites = XmCreateToggleButton ( form11, "new_suites", al, ac );

        ac = 0;
        form12 = XmCreateForm ( form10, "form12", al, ac );
        XtSetArg(al[ac], XmNindicatorType, XmONE_OF_MANY); ac++;
        XtSetArg(al[ac], XmNset, FALSE); ac++;
        toggle10 = XmCreateToggleButtonGadget ( form12, "@direct_read", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNorientation, XmVERTICAL); ac++;
	separator9 = XmCreateSeparator ( form12, "separator9", al, ac );
        ac = 0;
        XtSetArg(al[ac], XmNalignment, XmALIGNMENT_BEGINNING); ac++;
        direct_read = XmCreateToggleButton ( form12, "direct_read", al, ac );

        /* 201404 */
        ac = 0;
        form21 = XmCreateForm ( form10, "form21", al, ac );
        XtSetArg(al[ac], XmNindicatorType, XmONE_OF_MANY); ac++;
        XtSetArg(al[ac], XmNset, FALSE); ac++;
        toggle20 = XmCreateToggleButtonGadget ( form21, "@jobfile_length", al, ac );
        ac = 0;
        XtSetArg(al[ac], XmNorientation, XmVERTICAL); ac++;
        separator20 = XmCreateSeparator ( form21, "separator20", al, ac );

        ac = 0;
        XtSetArg(al[ac], XmNalignment, XmALIGNMENT_BEGINNING); ac++;
        label20 = XmCreateLabel ( form21, "Job file maximum length", al, ac );
        ac = 0;
        jobfile_length = XmCreateTextField ( form21, "jobfile_length", al, ac );
        XtSetArg(al[ac], XmNalignment, XmALIGNMENT_BEGINNING); ac++;
        label21 = XmCreateLabel ( form21, "lines", al, ac );
        /* */

        ac = 0;
        frame2 = XmCreateFrame ( form_, "frame2", al, ac );
        XtSetArg(al[ac], XmNchildType, XmFRAME_TITLE_CHILD); ac++;
	label2 = XmCreateLabel ( frame2, "Get server status:", al, ac );
	ac = 0;
	form2 = XmCreateForm ( frame2, "form2", al, ac );
	form4 = XmCreateForm ( form2, "form4", al, ac );
	XtSetArg(al[ac], XmNindicatorType, XmONE_OF_MANY); ac++;
	XtSetArg(al[ac], XmNset, FALSE); ac++;
	toggle2 = XmCreateToggleButtonGadget ( form4, "@poll", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNorientation, XmVERTICAL); ac++;
	separator7 = XmCreateSeparator ( form4, "separator7", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNalignment, XmALIGNMENT_BEGINNING); ac++;
	poll = XmCreateToggleButton ( form4, "poll", al, ac );
	ac = 0;
	form6 = XmCreateForm ( form2, "form6", al, ac );
	XtSetArg(al[ac], XmNindicatorType, XmONE_OF_MANY); ac++;
	XtSetArg(al[ac], XmNset, FALSE); ac++;
	toggle6 = XmCreateToggleButtonGadget ( form6, "@timeout", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNorientation, XmVERTICAL); ac++;
	separator4 = XmCreateSeparator ( form6, "separator4", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNalignment, XmALIGNMENT_BEGINNING); ac++;
	label3 = XmCreateLabel ( form6, "Call server every", al, ac );
	ac = 0;
	timeout = XmCreateTextField ( form6, "timeout", al, ac );
	XtSetArg(al[ac], XmNalignment, XmALIGNMENT_BEGINNING); ac++;
	label5 = XmCreateLabel ( form6, "seconds", al, ac );
	ac = 0;
	form5 = XmCreateForm ( form2, "form5", al, ac );
	XtSetArg(al[ac], XmNindicatorType, XmONE_OF_MANY); ac++;
	XtSetArg(al[ac], XmNset, FALSE); ac++;
	toggle7 = XmCreateToggleButtonGadget ( form5, "@drift", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNorientation, XmVERTICAL); ac++;
	separator5 = XmCreateSeparator ( form5, "separator5", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNalignment, XmALIGNMENT_BEGINNING); ac++;
	drift = XmCreateToggleButton ( form5, "drift", al, ac );
	ac = 0;
	form3 = XmCreateForm ( form2, "form3", al, ac );
	XtSetArg(al[ac], XmNindicatorType, XmONE_OF_MANY); ac++;
	XtSetArg(al[ac], XmNset, FALSE); ac++;
	toggle8 = XmCreateToggleButtonGadget ( form3, "@maximum", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNorientation, XmVERTICAL); ac++;
	separator6 = XmCreateSeparator ( form3, "separator6", al, ac );
	ac = 0;
	XtSetArg(al[ac], XmNalignment, XmALIGNMENT_BEGINNING); ac++;
	label4 = XmCreateLabel ( form3, "Don't reduce frequency over", al, ac );
	ac = 0;
	maximum = XmCreateTextField ( form3, "maximum", al, ac );
	XtSetArg(al[ac], XmNalignment, XmALIGNMENT_BEGINNING); ac++;
	label6 = XmCreateLabel ( form3, "minutes", al, ac );
	ac = 0;
	label7 = XmCreateLabelGadget ( form_, "warn2", al, ac );

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_POSITION); ac++;
	XtSetArg(al[ac], XmNtopPosition, 0); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_POSITION); ac++;
	XtSetArg(al[ac], XmNbottomPosition, 100); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_POSITION); ac++;
	XtSetArg(al[ac], XmNleftPosition, 0); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_POSITION); ac++;
	XtSetArg(al[ac], XmNrightPosition, 100); ac++;
	XtSetValues ( form_,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopOffset, 5); ac++;
	XtSetArg(al[ac], XmNtopWidget, frame2); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftOffset, 5); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightOffset, 5); ac++;
	XtSetValues ( frame1,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopOffset, 5); ac++;
	XtSetArg(al[ac], XmNtopWidget, frame1); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftOffset, 5); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightOffset, 5); ac++;
	XtSetValues ( frame3,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopOffset, 5); ac++;
	XtSetArg(al[ac], XmNtopWidget, label7); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftOffset, 5); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightOffset, 5); ac++;
	XtSetValues ( frame2,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNtopOffset, 5); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftOffset, 5); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightOffset, 5); ac++;
	XtSetValues ( label7,al, ac );
	ac = 0;


	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( form7,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget, form7); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( form8,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget, form8); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( form9,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget, form9); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( form13,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget, form13); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( form14,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget, form14); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( form15,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( toggle1,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNtopOffset, 4); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftWidget, toggle1); ac++;
	XtSetValues ( separator3,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftWidget, separator3); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( aborted,al, ac );
	ac = 0;
	XtAddCallback (toggle1, XmNvalueChangedCallback,&option_form_c:: useCB, (XtPointer) this);
	XtAddCallback (aborted, XmNvalueChangedCallback,&option_form_c:: changedCB, (XtPointer) this);
	children[ac++] = toggle1;
	children[ac++] = separator3;
	children[ac++] = aborted;
	XtManageChildren(children, ac);
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( toggle4,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNtopOffset, 4); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftWidget, toggle4); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_NONE); ac++;
	XtSetValues ( separator1,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftWidget, separator1); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( restarted,al, ac );
	ac = 0;
	XtAddCallback (toggle4, XmNvalueChangedCallback,&option_form_c:: useCB, (XtPointer) this);
	XtAddCallback (restarted, XmNvalueChangedCallback,&option_form_c:: changedCB, (XtPointer) this);
	children[ac++] = toggle4;
	children[ac++] = separator1;
	children[ac++] = restarted;
	XtManageChildren(children, ac);
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( toggle5,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNtopOffset, 4); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftWidget, toggle5); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_NONE); ac++;
	XtSetValues ( separator2,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftWidget, separator2); ac++;
        XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
        XtSetValues ( late,al, ac );
        ac = 0;
        XtAddCallback (toggle5, XmNvalueChangedCallback,
                       &option_form_c:: useCB, (XtPointer) this);
        XtAddCallback (late, XmNvalueChangedCallback,
                       &option_form_c:: changedCB, (XtPointer) this);
        children[ac++] = toggle5;
        children[ac++] = separator2;
        children[ac++] = late;
	XtManageChildren(children, ac);

	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( toggle13,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNtopOffset, 4); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftWidget, toggle13); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_NONE); ac++;
	XtSetValues ( separator13,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftWidget, separator13); ac++;
        XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
        XtSetValues ( zombied,al, ac );
        ac = 0;
        XtAddCallback (toggle13, XmNvalueChangedCallback,
                       &option_form_c:: useCB, (XtPointer) this);
        XtAddCallback (zombied, XmNvalueChangedCallback,
                       &option_form_c:: changedCB, (XtPointer) this);
        children[ac++] = toggle13;
        children[ac++] = separator13;
        children[ac++] = zombied;
	XtManageChildren(children, ac);

        
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( toggle14,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNtopOffset, 4); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftWidget, toggle14); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_NONE); ac++;
	XtSetValues ( separator14,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftWidget, separator14); ac++;
        XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
        XtSetValues ( aliases,al, ac );
        ac = 0;
        XtAddCallback (toggle14, XmNvalueChangedCallback,
                       &option_form_c:: useCB, (XtPointer) this);
        XtAddCallback (aliases,  XmNvalueChangedCallback,
                       &option_form_c:: changedCB, (XtPointer) this);

        children[ac++] = toggle14;
        children[ac++] = separator14;
	children[ac++] = aliases;
	XtManageChildren(children, ac);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( toggle15,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNtopOffset, 4); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftWidget, toggle15); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_NONE); ac++;
	XtSetValues ( separator15,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftWidget, separator15); ac++;
        XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
        XtSetValues ( late_family,al, ac );
        ac = 0;
        XtAddCallback (toggle15, XmNvalueChangedCallback,
                       &option_form_c:: useCB, (XtPointer) this);
        XtAddCallback (late_family,  XmNvalueChangedCallback,
                       &option_form_c:: changedCB, (XtPointer) this);

        children[ac++] = toggle15;
        children[ac++] = separator15;
	children[ac++] = late_family;
	XtManageChildren(children, ac);

	ac = 0;
	children[ac++] = form7;
	children[ac++] = form8;
	children[ac++] = form9;
	children[ac++] = form13;
	children[ac++] = form14;
	children[ac++] = form15;
	XtManageChildren(children, ac);
	ac = 0;
	children[ac++] = label1;
	children[ac++] = form1;
	XtManageChildren(children, ac);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( form11,al, ac );
        ac = 0;

        XtSetArg(al[ac], XmNtopAttachment, XmATTACH_WIDGET); ac++;
        XtSetArg(al[ac], XmNtopWidget, form11); ac++;
        XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
        XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
        XtSetValues ( form12,al, ac );
        ac = 0;

        /* 201404 */
        XtSetArg(al[ac], XmNtopAttachment, XmATTACH_WIDGET); ac++;
        XtSetArg(al[ac], XmNtopWidget, form12); ac++;
        XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
        XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
        XtSetValues ( form21,al, ac );
        ac = 0;

        XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
        XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
        XtSetArg(al[ac], XmNbottomOffset, 0); ac++;
        XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
        XtSetValues ( toggle20,al, ac );
        ac = 0;

        XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
        XtSetArg(al[ac], XmNtopOffset, 4); ac++;
        XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
        XtSetArg(al[ac], XmNbottomOffset, 4); ac++;
        XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
        XtSetArg(al[ac], XmNleftOffset, 4); ac++;
        XtSetArg(al[ac], XmNleftWidget, toggle20); ac++;
        XtSetArg(al[ac], XmNrightAttachment, XmATTACH_NONE); ac++;
        XtSetValues ( separator20,al, ac );
        ac = 0;

        XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
        XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
        XtSetArg(al[ac], XmNbottomOffset, 0); ac++;
        XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
        XtSetArg(al[ac], XmNleftOffset, 4); ac++;
        XtSetArg(al[ac], XmNleftWidget, separator20); ac++;
        XtSetArg(al[ac], XmNrightAttachment, XmATTACH_NONE); ac++;
        XtSetValues ( label20,al, ac );
        ac = 0;

        XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
        XtSetArg(al[ac], XmNtopOffset, 0); ac++;
        XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
        XtSetArg(al[ac], XmNbottomOffset, 0); ac++;
        XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
        XtSetArg(al[ac], XmNleftOffset, 0); ac++;
        XtSetArg(al[ac], XmNleftWidget, label20); ac++;
        XtSetArg(al[ac], XmNrightAttachment, XmATTACH_WIDGET); ac++;
        XtSetArg(al[ac], XmNrightOffset, 0); ac++;
        XtSetArg(al[ac], XmNrightWidget, label21); ac++;
        XtSetValues ( jobfile_length,al, ac );
        ac = 0;

        XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
        XtSetArg(al[ac], XmNtopOffset, 0); ac++;
        XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
        XtSetArg(al[ac], XmNbottomOffset, 0); ac++;
        XtSetArg(al[ac], XmNleftAttachment, XmATTACH_NONE); ac++;
        XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
        XtSetArg(al[ac], XmNrightOffset, 0); ac++;
        XtSetValues ( label21,al, ac );
        ac = 0;
        XtAddCallback (toggle20, XmNvalueChangedCallback,
                       &option_form_c:: useCB, (XtPointer) this);
        XtAddCallback (jobfile_length, XmNactivateCallback,
                       &option_form_c:: changedCB, (XtPointer) this);
        children[ac++] = toggle20;
        children[ac++] = separator20;
        children[ac++] = label20;
        children[ac++] = jobfile_length;
        children[ac++] = label21;
        XtManageChildren(children, ac);
        ac = 0;
        /* */

        XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
        XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( toggle3,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNtopOffset, 4); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftWidget, toggle3); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_NONE); ac++;
	XtSetValues ( separator8,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftWidget, separator8); ac++;
        XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
        XtSetValues ( new_suites,al, ac );
        ac = 0;
        XtAddCallback (toggle3, XmNvalueChangedCallback,
                       &option_form_c:: useCB, (XtPointer) this);
        XtAddCallback (new_suites, XmNvalueChangedCallback,
                       &option_form_c:: changedCB, (XtPointer) this);
        children[ac++] = toggle3;
        children[ac++] = separator8;
        children[ac++] = new_suites;
	XtManageChildren(children, ac);
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( toggle10,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNtopOffset, 4); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftWidget, toggle10); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_NONE); ac++;
	XtSetValues ( separator9,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftWidget, separator9); ac++;
        XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
        XtSetValues ( direct_read,al, ac );
        ac = 0;
        XtAddCallback (toggle10, XmNvalueChangedCallback,
                       &option_form_c:: useCB, (XtPointer) this);
        XtAddCallback (direct_read, XmNvalueChangedCallback,
                       &option_form_c:: changedCB, (XtPointer) this);
        children[ac++] = toggle10;
        children[ac++] = separator9;
        children[ac++] = direct_read;
        XtManageChildren(children, ac);
        ac = 0;
        children[ac++] = form11;
        children[ac++] = form12;
        children[ac++] = form21;
        XtManageChildren(children, ac);
	ac = 0;
	children[ac++] = label8;
	children[ac++] = form10;
	XtManageChildren(children, ac);
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNtopOffset, 0); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftOffset, 0); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( form4,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopOffset, 0); ac++;
	XtSetArg(al[ac], XmNtopWidget, form4); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftOffset, 0); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( form6,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopOffset, 0); ac++;
	XtSetArg(al[ac], XmNtopWidget, form6); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftOffset, 0); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( form5,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopOffset, 0); ac++;
	XtSetArg(al[ac], XmNtopWidget, form5); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftOffset, 0); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( form3,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomOffset, 0); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( toggle2,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNtopOffset, 4); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftWidget, toggle2); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_NONE); ac++;
	XtSetValues ( separator7,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftWidget, separator7); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( poll,al, ac );
	ac = 0;
	XtAddCallback (toggle2, XmNvalueChangedCallback,&option_form_c:: useCB, (XtPointer) this);
	XtAddCallback (poll, XmNvalueChangedCallback,&option_form_c:: changedCB, (XtPointer) this);
	children[ac++] = toggle2;
	children[ac++] = separator7;
	children[ac++] = poll;
	XtManageChildren(children, ac);
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( toggle6,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNtopOffset, 4); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftWidget, toggle6); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_NONE); ac++;
	XtSetValues ( separator4,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftWidget, separator4); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_NONE); ac++;
	XtSetValues ( label3,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftWidget, label3); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNrightWidget, label5); ac++;
	XtSetValues ( timeout,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( label5,al, ac );
	ac = 0;
	XtAddCallback (toggle6, XmNvalueChangedCallback,&option_form_c:: useCB, (XtPointer) this);
	XtAddCallback (timeout, XmNactivateCallback,&option_form_c:: changedCB, (XtPointer) this);
	children[ac++] = toggle6;
	children[ac++] = separator4;
	children[ac++] = label3;
	children[ac++] = timeout;
	children[ac++] = label5;
	XtManageChildren(children, ac);
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomOffset, 0); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( toggle7,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNtopOffset, 4); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftWidget, toggle7); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_NONE); ac++;
	XtSetValues ( separator5,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNtopOffset, 0); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomOffset, 0); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftWidget, separator5); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightOffset, 0); ac++;
	XtSetValues ( drift,al, ac );
	ac = 0;
	XtAddCallback (toggle7, XmNvalueChangedCallback,&option_form_c:: useCB, (XtPointer) this);
	XtAddCallback (drift, XmNvalueChangedCallback,&option_form_c:: changedCB, (XtPointer) this);
	children[ac++] = toggle7;
	children[ac++] = separator5;
	children[ac++] = drift;
	XtManageChildren(children, ac);
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomOffset, 0); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
	XtSetValues ( toggle8,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNtopOffset, 4); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftWidget, toggle8); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_NONE); ac++;
	XtSetValues ( separator6,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomOffset, 0); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 4); ac++;
	XtSetArg(al[ac], XmNleftWidget, separator6); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_NONE); ac++;
	XtSetValues ( label4,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNtopOffset, 0); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomOffset, 0); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftOffset, 0); ac++;
	XtSetArg(al[ac], XmNleftWidget, label4); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNrightOffset, 0); ac++;
	XtSetArg(al[ac], XmNrightWidget, label6); ac++;
	XtSetValues ( maximum,al, ac );
	ac = 0;

	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNtopOffset, 0); ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomOffset, 0); ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightOffset, 0); ac++;
	XtSetValues ( label6,al, ac );
	ac = 0;
	XtAddCallback (toggle8, XmNvalueChangedCallback,&option_form_c:: useCB, (XtPointer) this);
	XtAddCallback (maximum, XmNactivateCallback,&option_form_c:: changedCB, (XtPointer) this);
	children[ac++] = toggle8;
	children[ac++] = separator6;
	children[ac++] = label4;
	children[ac++] = maximum;
	children[ac++] = label6;
	XtManageChildren(children, ac);
	ac = 0;
	children[ac++] = form4;
	children[ac++] = form6;
	children[ac++] = form5;
	children[ac++] = form3;
	XtManageChildren(children, ac);
	ac = 0;
	children[ac++] = label2;
	children[ac++] = form2;
	XtManageChildren(children, ac);
	ac = 0;
	children[ac++] = frame1;
	children[ac++] = frame3;
	children[ac++] = frame2;
	children[ac++] = label7;
	XtManageChildren(children, ac);
	ac = 0;
	children[ac++] = form_;
	XtManageChildren(children, ac);
}

void option_form_c::useCB( Widget widget, XtPointer client_data, XtPointer call_data )
{
	option_form_p instance = (option_form_p) client_data;
	instance->useCB ( widget, call_data );
}

void option_form_c::changedCB( Widget widget, XtPointer client_data, XtPointer call_data )
{
	option_form_p instance = (option_form_p) client_data;
	instance->changedCB ( widget, call_data );
}

