/*
** Generated by X-Designer
*/
#ifndef _uitimetable_h
#define _uitimetable_h

#define XD_MOTIF

#include <xdclass.h>

class timetable_form_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget timetable_form;
	Widget file_;
	Widget from_;
	Widget to_;
	Widget all_;
	Widget tasks_;
	Widget by_name_;
	Widget by_time_;
	Widget time_;
	Widget menu_;
	Widget label_;
	Widget set_from_;
	Widget set_to_;
	Widget set_both_;
	Widget tools_;
public:
	static void optionsCB( Widget, XtPointer, XtPointer );
	virtual void optionsCB( Widget, XtPointer ) = 0;
	static void resetCB( Widget, XtPointer, XtPointer );
	virtual void resetCB( Widget, XtPointer ) = 0;
	static void setBothCB( Widget, XtPointer, XtPointer );
	virtual void setBothCB( Widget, XtPointer ) = 0;
	static void setToCB( Widget, XtPointer, XtPointer );
	virtual void setToCB( Widget, XtPointer ) = 0;
	static void setFromCB( Widget, XtPointer, XtPointer );
	virtual void setFromCB( Widget, XtPointer ) = 0;
	static void activateCB( Widget, XtPointer, XtPointer );
	virtual void activateCB( Widget, XtPointer ) = 0;
	static void updateCB( Widget, XtPointer, XtPointer );
	virtual void updateCB( Widget, XtPointer ) = 0;
	static void loadCB( Widget, XtPointer, XtPointer );
	virtual void loadCB( Widget, XtPointer ) = 0;
	static void mergeCB( Widget, XtPointer, XtPointer );
	virtual void mergeCB( Widget, XtPointer ) = 0;
};

typedef timetable_form_c *timetable_form_p;
class timetable;
typedef timetable *timetable_p;


extern timetable_p timetable_form;


#endif
