/*
** Generated by X-Designer
*/
#ifndef _uiedit_meter_h
#define _uiedit_meter_h

#define XD_MOTIF

#include <xdclass.h>

class edit_meter_form_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget edit_meter_form;
	Widget min_;
	Widget value_;
	Widget max_;
	Widget threshold_;
	Widget tools_;
public:
	static void changedCB( Widget, XtPointer, XtPointer );
	virtual void changedCB( Widget, XtPointer ) = 0;
	static void applyCB( Widget, XtPointer, XtPointer );
	virtual void applyCB( Widget, XtPointer ) = 0;
};

typedef edit_meter_form_c *edit_meter_form_p;
class edit_meter;
typedef edit_meter *edit_meter_p;


extern edit_meter_p edit_meter_form;


#endif
