/*
** Generated by X-Designer
*/
#ifndef _uivariables_h
#define _uivariables_h

#define XD_MOTIF

#include <xdclass.h>

class variables_form_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget variables_form;
	Widget list_;
	Widget edit_;
	Widget set_;
	Widget delete_;
	Widget name_;
	Widget value_;
	Widget find_;
public:
	static void valueCB( Widget, XtPointer, XtPointer );
	virtual void valueCB( Widget, XtPointer ) = 0;
	static void nameCB( Widget, XtPointer, XtPointer );
	virtual void nameCB( Widget, XtPointer ) = 0;
	static void deleteCB( Widget, XtPointer, XtPointer );
	virtual void deleteCB( Widget, XtPointer ) = 0;
	static void setCB( Widget, XtPointer, XtPointer );
	virtual void setCB( Widget, XtPointer ) = 0;
	static void browseCB( Widget, XtPointer, XtPointer );
	virtual void browseCB( Widget, XtPointer ) = 0;
	static void findCB( Widget, XtPointer, XtPointer );
	virtual void findCB( Widget, XtPointer ) = 0;
};

typedef variables_form_c *variables_form_p;
class variables;
typedef variables *variables_p;


extern variables_p variables_form;


#endif
