/*
** Generated by X-Designer
*/
#ifndef _uioutput_h
#define _uioutput_h

#define XD_MOTIF

#include <xdclass.h>

class output_form_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget output_form;
	Widget name_;
	Widget text_;
	Widget list_;
	Widget tools_;
public:
	static void searchCB( Widget, XtPointer, XtPointer );
	virtual void searchCB( Widget, XtPointer ) = 0;
	static void externalCB( Widget, XtPointer, XtPointer );
	virtual void externalCB( Widget, XtPointer ) = 0;
	static void updateCB( Widget, XtPointer, XtPointer );
	virtual void updateCB( Widget, XtPointer ) = 0;
	static void browseCB( Widget, XtPointer, XtPointer );
	virtual void browseCB( Widget, XtPointer ) = 0;
};

typedef output_form_c *output_form_p;
class output_panel;
typedef output_panel *output_panel_p;


extern output_panel_p output_form;


#endif
