/*
** Generated by X-Designer
*/
#ifndef _uiinfo_h
#define _uiinfo_h

#define XD_MOTIF

#include <xdclass.h>

class info_form_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget info_form;
	Widget text_;
public:
	static void hyperCB( Widget, XtPointer, XtPointer );
	virtual void hyperCB( Widget, XtPointer ) = 0;
};

typedef info_form_c *info_form_p;


extern info_form_p info_form;


#endif
