/*
** Generated by X-Designer
*/
#ifndef _uiedit_variable_h
#define _uiedit_variable_h

#define XD_MOTIF

#include <xdclass.h>

class edit_variable_form_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget edit_variable_form;
	Widget tools_;
	Widget name_;
	Widget value_;
public:
	static void changedCB( Widget, XtPointer, XtPointer );
	virtual void changedCB( Widget, XtPointer ) = 0;
	static void applyCB( Widget, XtPointer, XtPointer );
	virtual void applyCB( Widget, XtPointer ) = 0;
};

typedef edit_variable_form_c *edit_variable_form_p;
class edit_variable;
typedef edit_variable *edit_variable_p;


extern edit_variable_p edit_variable_form;


#endif
