/*
** Generated by X-Designer
*/
#ifndef _uihistory_h
#define _uihistory_h

#define XD_MOTIF

#include <xdclass.h>

class history_form_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget history_form;
	Widget list_;
public:
	static void browseCB( Widget, XtPointer, XtPointer );
	virtual void browseCB( Widget, XtPointer ) = 0;
};

typedef history_form_c *history_form_p;
class history;
typedef history *history_p;


extern history_p history_form;


#endif
