suite s1
 clock real 300 -s    #  
 endclock  400  #  
endsuite
 
suite s3
 clock real 20.1.1992 300 -s  
 endclock 23.1.1992 300 
endsuite

suite s4
 clock real 20.1.1992 +01:00 -s
endsuite

suite s5
 clock real 300  -s  # -s means stop start calendar with server
endsuite

suite s6
 clock real +01:00  -s # 
endsuite

suite s7
 clock real 20.1.1992 +01:00   -s # comment
endsuite