/*
** Generated by X-Designer
*/
#ifndef _uitip_h
#define _uitip_h

#define XD_MOTIF

#include <xdclass.h>

class tip_shell_c: public xd_TopLevelShell_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget tip_shell;
	Widget form_;
	Widget label_;
};

typedef tip_shell_c *tip_shell_p;


extern tip_shell_p tip_shell;


#endif
