/*
** Generated by X-Designer
*/
#ifndef _uiservers_h
#define _uiservers_h

#define XD_MOTIF

#include <xdclass.h>

class servers_form_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget servers_form;
	Widget form_;
	Widget list_;
	Widget add_;
	Widget update_;
	Widget remove_;
	Widget name_;
	Widget host_;
	Widget number_;
public:
	static void updateCB( Widget, XtPointer, XtPointer );
	virtual void updateCB( Widget, XtPointer ) = 0;
	static void changedCB( Widget, XtPointer, XtPointer );
	virtual void changedCB( Widget, XtPointer ) = 0;
	static void browseCB( Widget, XtPointer, XtPointer );
	virtual void browseCB( Widget, XtPointer ) = 0;
	static void removeCB( Widget, XtPointer, XtPointer );
	virtual void removeCB( Widget, XtPointer ) = 0;
	static void addCB( Widget, XtPointer, XtPointer );
	virtual void addCB( Widget, XtPointer ) = 0;
	static void serversCB( Widget, XtPointer, XtPointer );
	virtual void serversCB( Widget, XtPointer ) = 0;
};

typedef servers_form_c *servers_form_p;


extern servers_form_p servers_form;


#endif
