/*
** Generated by X-Designer
*/
#ifndef _uiconfirm_h
#define _uiconfirm_h

#define XD_MOTIF

#include <xdclass.h>

class confirm_shell_c: public xd_XmDialog_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget confirm_shell;
	Widget form_;
	Widget label_;
public:
	static void helpCB( Widget, XtPointer, XtPointer );
	virtual void helpCB( Widget, XtPointer ) = 0;
	static void cancelCB( Widget, XtPointer, XtPointer );
	virtual void cancelCB( Widget, XtPointer ) = 0;
	static void okCB( Widget, XtPointer, XtPointer );
	virtual void okCB( Widget, XtPointer ) = 0;
};

typedef confirm_shell_c *confirm_shell_p;


extern confirm_shell_p confirm_shell;


#endif
