/*
** Generated by X-Designer
*/
#ifndef _uitop_h
#define _uitop_h

#define XD_MOTIF

#include <xdclass.h>

class top_shell_c: public xd_ApplicationShell_c {
public:
	virtual void create (Display *display, char *app_name, int app_argc, char **app_argv, char *app_class_name = NULL);
protected:
	Widget top_shell;
	Widget form_;
	Widget message_;
	Widget tools_;
	Widget trees_;
	Widget menu_bar;
	Widget show0_;
	Widget show_unknown;
	Widget show_suspended;
	Widget show_complete;
	Widget show_queued;
	Widget show_submitted;
	Widget show_active;
	Widget show_aborted;
	Widget show_all; Widget show_none;
	Widget show1_;
	Widget show2_;
	Widget show3_;
	Widget show4_;
	Widget servers_menu_;
	Widget windows_menu_;
	Widget help;
	Widget snaphot;

#ifndef file_widget
#define file_menu cascade55
#endif

	Widget cascade55; // = (Widget)NULL;

public:
	static void loginCB( Widget, XtPointer, XtPointer );
	virtual void loginCB( Widget, XtPointer ) = 0;
	static void prefCB( Widget, XtPointer, XtPointer );
	virtual void prefCB( Widget, XtPointer ) = 0;
	static void helpCB( Widget, XtPointer, XtPointer );
	virtual void helpCB( Widget, XtPointer ) = 0;
	static void searchCB( Widget, XtPointer, XtPointer );
	virtual void searchCB( Widget, XtPointer ) = 0;
	static void windowsCB( Widget, XtPointer, XtPointer );
	virtual void windowsCB( Widget, XtPointer ) = 0;
	static void chatCB( Widget, XtPointer, XtPointer );
	virtual void chatCB( Widget, XtPointer ) = 0;
	static void showCB( Widget, XtPointer, XtPointer );
	virtual void showCB( Widget, XtPointer ) = 0;
	static void serverCB( Widget, XtPointer, XtPointer );
	virtual void serverCB( Widget, XtPointer ) = 0;
	static void statusCB( Widget, XtPointer, XtPointer );
	virtual void statusCB( Widget, XtPointer ) = 0;
	static void windowCB( Widget, XtPointer, XtPointer );
	virtual void windowCB( Widget, XtPointer ) = 0;
	static void quitCB( Widget, XtPointer, XtPointer );
	virtual void quitCB( Widget, XtPointer ) = 0;

	static void releaseCB( Widget, XtPointer, XtPointer );
	virtual void releaseCB( Widget, XtPointer ) = 0;
        static void snapshotCB( Widget, XtPointer, XtPointer );
        virtual void snapshotCB( Widget, XtPointer ) = 0;
};

typedef top_shell_c *top_shell_p;
class top;
typedef top *top_p;
extern top_p top_shell;
#endif
