/*
** Generated by X-Designer
*/
#ifndef _uitree_h
#define _uitree_h

#define XD_MOTIF

#include <xdclass.h>

class tree_c: public xd_XmDrawingArea_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget tree_;
	Widget see_menu_;
	Widget show_current_;
	Widget fold_around_;
	Widget hide_other_;
	Widget snapshot_;
	Widget why_menu_;
	Widget why_label_;
public:
	static void hideOtherCB( Widget, XtPointer, XtPointer );
	virtual void hideOtherCB( Widget, XtPointer ) = 0;
	static void aroundCB( Widget, XtPointer, XtPointer );
	virtual void aroundCB( Widget, XtPointer ) = 0;
	static void foldCB( Widget, XtPointer, XtPointer );
	virtual void foldCB( Widget, XtPointer ) = 0;
	static void unfoldCB( Widget, XtPointer, XtPointer );
	virtual void unfoldCB( Widget, XtPointer ) = 0;
        static void showCB( Widget, XtPointer, XtPointer );
        virtual void showCB( Widget, XtPointer ) = 0;

        static void snapshotCB( Widget, XtPointer, XtPointer );
        virtual void snapshotCB( Widget, XtPointer ) = 0;
};

typedef tree_c *tree_p;


extern tree_p tree_;


#endif
