/*
** Generated by X-Designer
*/
#ifndef _uiedit_h
#define _uiedit_h

#define XD_MOTIF

#include <xdclass.h>

class edit_form_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget edit_form;
	Widget text_;
	Widget tools_;
	Widget external_;
	Widget tools2_;
	Widget alias_;
	Widget preprocess_;
public:
	static void searchCB( Widget, XtPointer, XtPointer );
	virtual void searchCB( Widget, XtPointer ) = 0;
	static void preprocCB( Widget, XtPointer, XtPointer );
	virtual void preprocCB( Widget, XtPointer ) = 0;
	static void submitCB( Widget, XtPointer, XtPointer );
	virtual void submitCB( Widget, XtPointer ) = 0;
	static void externalCB( Widget, XtPointer, XtPointer );
	virtual void externalCB( Widget, XtPointer ) = 0;
	static void changedCB( Widget, XtPointer, XtPointer );
	virtual void changedCB( Widget, XtPointer ) = 0;
};

typedef edit_form_c *edit_form_p;
class edit;
typedef edit *edit_p;


extern edit_p edit_form;


#endif
