#ifndef confirm_H
#define confirm_H

//=============================================================================================
// Name        : 
// Author      : 
// Revision    : $Revision: #4 $ 
//
// Copyright 2009-2017 ECMWF.
// This software is licensed under the terms of the Apache Licence version 2.0 
// which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
// In applying this licence, ECMWF does not waive the privileges and immunities 
// granted to it by virtue of its status as an intergovernmental organisation 
// nor does it submit to any jurisdiction. 
//
// Description : 
//=============================================================================================

#include "uiconfirm.h"
#include "dialog.h"
#include "str.h"

class confirm : public dialog<confirm,confirm_shell_c> {
public:

	confirm();

	~confirm(); // Change to virtual if base class

	static Boolean ask(Boolean,const char*,...);
	static Boolean ask(Boolean,str& msg);

private:

	confirm(const confirm&);
	confirm& operator=(const confirm&);
};

inline void destroy(confirm**) {}
#endif
