/*
** Generated by X-Designer
*/
#ifndef _uitriggers_h
#define _uitriggers_h

#define XD_MOTIF

#include <xdclass.h>

class triggers_menu_c: public xd_XmCascadeButton_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget triggers_menu;
	Widget triggers_button_;
	Widget triggered_button_;
public:
	static void reachCB( Widget, XtPointer, XtPointer );
	virtual void reachCB( Widget, XtPointer ) = 0;
	static void fullCB( Widget, XtPointer, XtPointer );
	virtual void fullCB( Widget, XtPointer ) = 0;
	static void triggeredCB( Widget, XtPointer, XtPointer );
	virtual void triggeredCB( Widget, XtPointer ) = 0;
	static void triggersCB( Widget, XtPointer, XtPointer );
	virtual void triggersCB( Widget, XtPointer ) = 0;
};

typedef triggers_menu_c *triggers_menu_p;


class triggers_form_c: public xd_XmForm_c {
public:
	virtual void create (Widget parent, char *widget_name = NULL);
protected:
	Widget triggers_form;
	Widget tgraph_;
	Widget ttext_;
	Widget dependencies_button_;
	Widget graph_;
	Widget info_menu_;
	Widget link_menu_;
	Widget text_;
public:
	static void linkCB( Widget, XtPointer, XtPointer );
	virtual void linkCB( Widget, XtPointer ) = 0;
	static void hyperCB( Widget, XtPointer, XtPointer );
	virtual void hyperCB( Widget, XtPointer ) = 0;
	static void entryCB( Widget, XtPointer, XtPointer );
	virtual void entryCB( Widget, XtPointer ) = 0;
	static void dependCB( Widget, XtPointer, XtPointer );
	virtual void dependCB( Widget, XtPointer ) = 0;
};

typedef triggers_form_c *triggers_form_p;
class triggers;
typedef triggers *triggers_p;


extern triggers_menu_p triggers_menu;

extern triggers_p triggers_form;



#endif
