# Notify ECF of a normal end
%ECF_CLIENT_EXE_PATH% --complete
trap 0                          # Remove all traps
exit 0                          # End the shell

%manual
#This is the manual from the tail.h file
%end
