 
suite s3
   label foo ""
   label foo2 "value"
   label foo3 "value\\nvalue" #\\n should be converted to new line
   label foo4 "value\\nvalue  \\nfred fred2\\n" # \\n should be converted to new line
endsuite
